import os
from github import Github, GithubException
from oauth2client.client import OAuth2WebServerFlow
from oauth2client.contrib import keyring_storage
from oauth2client.tools import run_flow


CLIENT_ID = '1fe774845dda5a15936d'
CLIENT_SECRET = '5212960321bcd875538f2221769ae74612e27a04'
SCOPE = 'repo'
AUTH_URI = 'https://github.com/login/oauth/authorize'
TOKEN_URI = 'https://github.com/login/oauth/access_token'


class AuthException(Exception):
    def __init__(self, message):
        self.message = message


def run_oauth_flow():
    flow = OAuth2WebServerFlow(CLIENT_ID, CLIENT_SECRET, SCOPE, auth_uri=AUTH_URI, token_uri=TOKEN_URI)
    storage = _get_storage()
    cred = run_flow(flow, storage)

    verify_access_token()

    if cred is not None:
        return cred.access_token
    else:
        return None


def get_api():
    storage = _get_storage()
    cred = storage.get()
    if cred is None:
        raise AuthException('No local authorization found. Run \'auth\' to add one')

    token = cred.access_token
    api = Github(token)
    return api


def verify_access_token():
    try:
        api = get_api()
    except AuthException as exception:
        print('Error: {0}'.format(exception.message))
        return False

    user = api.get_user()

    try:
        print("Successfully authenticated as {0} (Name: '{1}' Email: '{2}')".format(user.login, user.name, user.email))
        return True
    except GithubException as exception:
        print('Failed with {0}, data: {1}'.format(exception.status, exception.data))
        print('Unable to verify local auth token. Run \'auth\' to reset')

    return False


def delete_access_token():
    storage = _get_storage()
    storage.delete()


def _get_storage():
    return keyring_storage.Storage('PlushKeys', 'default')



from fabric.api import run, sudo
from fabric.contrib.files import exists

from .permissions import make_directory, set_permissions_directory


DEFAULT_KEY_DIRECTORY = '/var/deploykeys'


def get_keyfile(project_name, directory=DEFAULT_KEY_DIRECTORY, public=True):
    sanitized_name = project_name.replace('/', '_')
    suffix = '.pub' if public else ''
    keyfile = '{0}/{1}{2}'.format(directory, sanitized_name, suffix)
    return keyfile


def create_key(project_name, owning_group, directory=DEFAULT_KEY_DIRECTORY):
    make_directory(owning_group, directory, mod='600')
    keyfile = get_keyfile(project_name, directory, public=False)
    if exists(keyfile, use_sudo=True):
        sudo('rm {0}'.format(keyfile))
    sudo('ssh-keygen -t rsa -b 4096 -C "{0}" -f {1}'.format(project_name, keyfile))
    set_permissions_directory(directory, owning_group, mod='600')



from fabric.api import run, sudo
from github import GithubException
from .oauth_flow import get_api
from .fabric_commands import get_keyfile


def list_repo_keys(repo_full_name):
    api = get_api()

    repo = api.get_repo(repo_full_name)
    deploy_keys = repo.get_keys()

    try:
        for key in deploy_keys:
            print(key.title)
    except GithubException as exception:
        print('Failed with {0}, data: {1}'.format(exception.status, exception.data))


def add_repo_key(repo_full_name, key_name=''):
    public_keyfile = get_keyfile(repo_full_name)
    public_keyfile_contents = sudo('cat {0}'.format(public_keyfile))

    if key_name == '':
        # Default the key_name to the hostname we are connected to
        key_name = run('hostname')

    api = get_api()

    repo = api.get_repo(repo_full_name)

    try:
        repo.create_key(key_name, public_keyfile_contents, read_only=True)
    except GithubException as exception:
        print('Failed with {0}, data: {1}'.format(exception.status, exception.data))



import setuptools

long_description="""Plush is designed to SSH deploy keys on a remote computer using Fabric. These deploy keys are stored in the repo settings for your repo on GitHub using the GitHub API (using OAUTH credentials). Plush then clones your repo using a remote path that references these deploy keys. Plush allows multiple projects to have unique deploy keys all on one computer."""

setuptools.setup(name='plush-fabric',
      version='0.4.0',
      description='Helper library for Fabric to simplify creating and managing GitHub deploy keys when deploying GitHub-hosted repositories',
      long_description=long_description,
      author='Kevin Barnes',
      author_email='kbarnes3@gmail.com',
      url='https://github.com/kbarnes3/Plush',
      license='BSD',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Topic :: Software Development',
          'License :: OSI Approved :: BSD License',
          "Operating System :: OS Independent",
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.6',
          ],
      keywords='fabric github deployment',
      install_requires=['oauth2client>=3.0.0', 'PyGithub>=1.3.8', 'keyring>=12.0.0'],
      packages=['plush', 'plush.fabric_commands'],
      entry_points={
          'console_scripts': [
              'auth = plush.console:auth_entry',
              'listkeys = plush.console:list_keys_entry'
          ]
      },
      )



from fabric.api import settings, sudo
from fabric.contrib.files import exists
from .ssh_key import get_keyfile


class _AllowedException(Exception):
    pass


def prepare_user(user, group, add_sudo=True, no_sudo_passwd=False):
    messages = ''
    user_exists = False

    with settings(abort_exception=_AllowedException):
        try:
            sudo('getent passwd {0}'.format(user))
            user_exists = True
        except _AllowedException:
            pass

    if not user_exists:
        sudo('adduser --disabled-password {0}'.format(user))

    group_exists = False
    with settings(abort_exception=_AllowedException):
        try:
            sudo('getent group {0}'.format(group))
            group_exists = True
        except _AllowedException:
            pass

    if not group_exists:
        sudo('addgroup {0}'.format(group))

    sudo('adduser {0} {1}'.format(user, group))

    if add_sudo:
        sudo('adduser {0} sudo'.format(user))

    if no_sudo_passwd:
        sudoers_file = '/etc/sudoers.d/{0}-plush'.format(user)
        if exists(sudoers_file, True):
            sudo('rm {0}'.format(sudoers_file))
        sudo("echo '{0} ALL=(ALL:ALL) NOPASSWD:ALL' | sudo EDITOR='tee -a' visudo -f {1}".format(user, sudoers_file))

    if add_sudo and not no_sudo_passwd:
        messages += 'Ensure {0} has a secure password configured in order to run commands as sudo.\n'.format(user)
        messages += 'Alternatively, rerun this command with the no_sudo_passwd=True parameter\n'
    if not user_exists:
        messages += 'The {0} account was created without a password set. A superuser will need to manually run ' \
                    '\"sudo passwd {0}\"\n'.format(user)
        messages += 'Where possible, consider not setting a password and instead using key based authentication.\n'

    if messages:
        # Every message line ends in a newline. If we've got a non-empty message, strip the last character
        messages = messages[:-1]

    return messages


def add_authorized_key(user, public_key):
    sudo('mkdir -p ~/.ssh', user=user)
    sudo('touch ~/.ssh/authorized_keys', user=user)
    sudo('chmod -R go= ~/.ssh ', user=user)
    sudo('echo "{0}" | tee -a ~/.ssh/authorized_keys'.format(public_key), user=user)



from fabric.api import sudo
from .ssh_key import get_keyfile


def clone(repo_full_name, path, skip_strict_key_checking=False):
    keyfile = get_keyfile(repo_full_name, public=False)
    sudo('git config --system protocol.ext.allow always')
    strict_key_checking = '-o StrictHostKeyChecking=no ' if skip_strict_key_checking else ''
    sudo("git clone 'ext::ssh -i {0} {1}git@github.com %S {2}.git' {3}".format(
        keyfile,
        strict_key_checking,
        repo_full_name,
        path))



from fabric.api import cd, sudo


def make_directory(owning_group, directory, mod='ug+rwX,o+rX,o-w'):
    sudo('mkdir -p {0}'.format(directory))
    set_permissions_directory(directory, group=owning_group, mod=mod)


def set_permissions_directory(directory, group=None, user=None, mod='660', setgid=True):
    with cd(directory):
        if group is not None:
            sudo('chgrp -R {0} .'.format(group))

        if user is not None:
            sudo('chown -R {0} .'.format(user))

        sudo('chmod -R {0} .'.format(mod))

        if setgid:
            sudo('chmod -R g+s .')


def set_permissions_file(file, user=None, group=None, mod='644'):
    if group is not None:
        sudo('chgrp {0} {1}'.format(group, file))

    if user is not None:
        sudo('chown {0} {1}'.format(user, file))

    sudo('chmod {0} {1}'.format(mod, file))
